#property copyright ""
#property link      ""

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 RoyalBlue
#property indicator_color2 Crimson
#property indicator_width1  2
#property indicator_width2  2
#property indicator_maximum 1
#property indicator_minimum 0

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame = PERIOD_CURRENT;
extern int    period          = 10;
extern double mod             = 1.0;
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;
extern string soundFile       = "alert2.wav";
extern bool   linesVisible    = false;
extern string linesIdentifier = "ghl lines1";
extern color  linesUpColor    = DeepSkyBlue;
extern color  linesDnColor    = PaleVioletRed;
extern ENUM_LINE_STYLE LinesStyle = STYLE_DOT;
extern int    LinesWidth      = 0;

//
//
//
//
//

double up_buffer[];
double dn_buffer[];
double trend[];

string indicatorFileName;
bool   returnBars;

int init() 
{
   IndicatorBuffers(3); 
   SetIndexBuffer(0,up_buffer);  SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,dn_buffer);  SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,trend);
   
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == -99;
   TimeFrame         = MathMax(TimeFrame,_Period);
   IndicatorShortName(timeFrameToString(TimeFrame)+" Gann HiLo"); 
return(0);
}
int deinit()
{
   string lookFor       = linesIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   
   //
   //
   //
   //
   //
   
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
      if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { up_buffer[0] = limit+1; return(0); }
   
            if (TimeFrame!=Period())
            {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,TimeFrame,Time[i]);               
                      up_buffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,period,mod,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,linesVisible,linesIdentifier,linesUpColor,linesDnColor,LinesStyle,LinesWidth,0,y);
                      dn_buffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,period,mod,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,linesVisible,linesIdentifier,linesUpColor,linesDnColor,LinesStyle,LinesWidth,1,y);
               }
               return(0);
            }

   //
   //
   //
   //
   //
   
   for (i=limit; i>=0; i--)
   {
      trend[i]=trend[i+1];
         if (Close[i]>((1.0 + mod / 10000.0)*iMA(Symbol(),0,period,0,MODE_SMA,PRICE_CLOSE,i+1))) trend[i]= 1;
         if (Close[i]<((1.0 - mod / 10000.0)*iMA(Symbol(),0,period,0,MODE_SMA,PRICE_CLOSE,i+1))) trend[i]=-1;
    
         up_buffer[i] = EMPTY_VALUE;
         dn_buffer[i] = EMPTY_VALUE;
         if (trend[i]== 1) up_buffer[i] = 1;
         if (trend[i]==-1) dn_buffer[i] = 1;
         
         //
         //
         //
         //
         //
         
         if (linesVisible )
         {
           ObjectDelete(linesIdentifier+":"+Time[i]);
           if (trend[i]!= trend[i+1])
           {
              if (trend[i+1] ==-1 && trend[i] !=-1) drawLine(i,linesUpColor);
              if (trend[i+1] == 1 && trend[i] != 1) drawLine(i,linesDnColor);
           }
         }
  }
  
  //
  //
  //
  //
  //
      
  if (alertsOn)
  {
    if (alertsOnCurrent)
         int whichBar = 0;
    else     whichBar = 1;
    if (trend[whichBar] != trend[whichBar+1])
    if (trend[whichBar] == 1)
          doAlert("Buy");
    else  doAlert("Sell"); 
  }      
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," GannHiLo-Histo ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," GannHiLo-Histo "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//
//
//
//
//

void drawLine(int i,color theColor)
{
     string name = linesIdentifier+":"+Time[i];
   
     //
     //
     //
     //
     //
      
     ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
        ObjectSet(name,OBJPROP_COLOR,theColor);
        ObjectSet(name,OBJPROP_STYLE,LinesStyle);
        ObjectSet(name,OBJPROP_WIDTH,LinesWidth);
        ObjectSet(name,OBJPROP_BACK,true);
}